package com.jtauber.river.parts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;

import com.jtauber.river.figures.NodeFigure;
import com.jtauber.river.model.Node;

public class NodePart extends AbstractGraphicalEditPart implements PropertyChangeListener {

   //------------------------------------------------------------------------
   // Overridden from EditPart

   // override activate to register with the model for property changes
   public void activate() {
      if (isActive()) {
         return;
      }
      super.activate();
      ((Node) getModel()).addPropertyChangeListener(this);
   }

   // override deactivate to deregister with the model
   public void deactivate() {
      if (!isActive()) {
         return;
      }
      super.deactivate();
      ((Node) getModel()).removePropertyChangeListener(this);
   }


   //------------------------------------------------------------------------
   // Overridden from AbstractEditPart

   protected void refreshVisuals() {
      Node node = (Node) this.getModel();
      Point loc = node.getLocation();
      Dimension size = new Dimension(150, 40);
      Rectangle rectangle = new Rectangle(loc, size);

      // tells the parent part (in this case DiagramPart) that this part
      // and its figure are to be constrained to the given rectangle
      ((GraphicalEditPart) getParent()).setLayoutConstraint(this, getFigure(), rectangle);
   }


   //------------------------------------------------------------------------
   // Abstract methods from AbstractGraphicalEditPart

   protected IFigure createFigure() {
      return new NodeFigure();
   }


   //------------------------------------------------------------------------
   // Abstract methods from AbstractEditPart

   protected void createEditPolicies() {
      // none    
   }


   //------------------------------------------------------------------------
   // Abstract methods from PropertyChangeListener

   // catch changes to LOCATION and refresh visuals for this part
   public void propertyChange(PropertyChangeEvent changeEvent) {
      if (changeEvent.getPropertyName().equals(Node.LOCATION))
         refreshVisuals();
   }
}